/****************************************************************/
/*  
     ***   ***  ***  *** ***     *** ***   ******  ****  ***
     ***   ***  ***  *** ***     *** **   ***  *** ***** ***
     ***   ***  ***  *** ***     *****    ******** *** *****
      *** ***   ***  *** ******* *** **   ***  *** ***  ****
       *****     *****   ******* *** ***  ***  *** ***   ***
	   
	 	 Vulkan Graphics API: 11 Steps

     Minimalistic examples to build upon and experiment with 
	 to develop an understanding of the underlying API

	   
/*
/****************************************************************/
/*
     About:
	 Self contained demo (i.e., each section encapsulated within
	 a function - see the parameter dependencies). 
	 Remember to include and compile the two binary shader
	 files (vertex and shader)

	 ''''
	 #define for enabling/disabling `DEPTH_BUFFER' - 
	 extension (additional code) is needed from the minimal
	 version in the original book to have a working
	 depth buffer (i.e., zbuffer).
	 ''''

*/


#pragma once


#define _CRT_SECURE_NO_WARNINGS // use fopen(..)

#include <windows.h>
#include <math.h>

#include <vector>
using namespace std;

// For variable argument functions ,e.g., dprintf(..), 
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>  // vsprintf_s


// Windows version of the Vulkan API
#define VK_USE_PLATFORM_WIN32_KHR

// this is if we do not declared prototypes, 
// and want to load dynamically!
//#define VK_NO_PROTOTYPES            
                                      
#include "vulkan.h"



#define DEPTH_BUFFER





//Saving debug information to a log file/screen/..
inline 
void dprintf(const char *fmt, ...) 
{
	va_list parms;
	static char buf[2048] = {0};

	// Try to print in the allocated space.
	va_start(parms, fmt);
	vsprintf_s(buf, fmt, parms);
	va_end(parms);

	// Write the information out to a txt file
	#if 0
	FILE *fp = fopen("output.txt", "a+");
	fprintf(fp, "%s",  buf);
	fclose(fp);
	#endif

	// Output to the visual studio window
	OutputDebugStringA( buf );

}// End dprintf(..)


// Debug defines (custom asserts)
#if defined(_WIN32)
  #define DBG_ASSERT(f) { if(!(f)){ __debugbreak(); }; }
#else
  #define DBG_ASSERT(f) { #error(platform assert todo) }
#endif

#define DBG_VALID(f)  { if( (f)!=(f) ){DBG_ASSERT(false);} }

#define DBG_ASSERT_MSG( val, errmsg )        \
	if(!(val)){                                \
	DBG_ASSERT( false )                        \
	}

#define DBG_ASSERT_VULKAN( val )             \
    DBG_ASSERT( VK_SUCCESS == val )

#define DBG_ASSERT_VULKAN_MSG( val, errmsg ) \
	if(!((VK_SUCCESS == val))){                \
    DBG_ASSERT( false )                      \
	}





// Win32 message callback
LRESULT CALLBACK WindowProc( HWND	hwnd, 
                             UINT	uMsg, 
                             WPARAM wParam, 
                             LPARAM lParam );


// Step-1-
void SetupWindow(int	width, 
				 int	height, 
				 HWND*	outWindowHandle);




// Step-2-
void SetupVulkanInstance(HWND				windowHandle,
					     VkInstance*		outInstance,
					     VkSurfaceKHR*		outSurface);





// Step-3-
void SetupPhysicalDevice(VkInstance			instance,
						 VkPhysicalDevice*	outPhysicalDevice,
             VkDevice*			outDevice);




// Step-4-
void SetupSwapChain(VkDevice			device,
					VkPhysicalDevice	physicalDevice,
					VkSurfaceKHR		surface,
					int*				outWidth,
					int*				outHeight,
					VkSwapchainKHR*		outSwapChain,			     
					VkImage**			outPresentImages,				     
					VkImageView**		outPresentImageViews);




// Step-5-
void SetupRenderPass(VkDevice			device,
					 VkPhysicalDevice	physicalDevice,
					 int				width,
					 int				height,
					 VkImageView*		presentImageViews,
					 VkRenderPass*		outRenderPass, 
					 VkFramebuffer**	outFrameBuffers);



// Step-6-
void SetupCommandBuffer(VkDevice			device,
						VkPhysicalDevice	physicalDevice,
						VkCommandBuffer*	outCommandBuffer);



// Step-7-
void SetupVertexBuffer(VkDevice			device,
					   VkPhysicalDevice	physicalDevice,
					   int*				outVertexSize,
					   int*				outNumberOfTriangles,
					   VkBuffer*		outVertexInputBuffer );





// Step-8-
void SetupShaderandUniforms(VkDevice			device,
							VkPhysicalDevice	physicalDevice,
							int					width,
							int					height,
							VkShaderModule*		outVertShaderModule,
							VkShaderModule*		outFragShaderModule,
              VkShaderModule*		outTescShaderModule,
							VkShaderModule*		outTeseShaderModule,
							VkBuffer*			outBuffer,
							VkDeviceMemory*		outMemory);


// Step-9-
void SetupDescriptors(VkDevice				 device,
                      VkPhysicalDevice physicalDevice,
					  VkBuffer				 buffer,
					  VkDescriptorSet*	 	 outDescriptorSet,
					  VkDescriptorSetLayout* outDescriptorSetLayout);






// Step-10-
void SetupPipeline(	VkDevice				device,
				    int						width,
					int						height,
					int						vertexSize,
					VkDescriptorSetLayout	descriptorSetLayout,
					VkShaderModule			vertShaderModule,
					VkShaderModule			fragShaderModule,
          VkShaderModule			tescShaderModule,
					VkShaderModule			teseShaderModule,
					VkRenderPass			renderPass, 
					VkPipeline*				outPipeline,
					VkPipelineLayout*		outPipelineLayout);




// Step-11-
void RenderLoop(VkDevice			device,
				int					width,
				int					height,
				int					numberOfTriangles,
				VkSwapchainKHR		swapChain,
				VkCommandBuffer		commandBuffer,
				VkImage*			presentImages,
				VkFramebuffer*		frameBuffers,
				VkRenderPass		renderPass,
				VkBuffer			vertexInputBuffer,
				VkDescriptorSet	 	descriptorSet,
				VkPipelineLayout	pipelineLayout,
				VkPipeline			pipeline);

















