

#pragma once


#include <vector>
using namespace std;


// Speaks for itself
#define MPI        3.14159265358979323846
#define TWOPI      6.28318530717958647692


struct ShapePoint
{
    float x,  y,  z;  // position
    float nx, ny, nz; // normal
    float u,  v;      // texture coordinates
};

inline
void GenerateTube(float radius, float height, vector<ShapePoint>* outTris) 
{
  radius = 2.5f;
  height = 2.5f;

  int numSteps = 12;
  int numRings = 12;
  for (int h = 0; h< numSteps-1; ++h)
  {
    for (int r = 0; r < numRings+1; ++r)
    {
        int r0 = r;
        int r1 = r+1;
        int h0 = h;
        int h1 = h+1;

        float angle = (float)TWOPI / (numRings);
        float x0 = sinf( r0 * angle ) * radius;
        float z0 = cosf( r0 * angle ) * radius;
        float y0 = (h0 / (numSteps-1.0f)) * height;

        float x1 = sinf( r1 * angle ) * radius;
        float z1 = cosf( r1 * angle ) * radius;
        float y1 = (h0 / (numSteps-1.0f)) * height;

        float x2 = sinf( r0 * angle ) * radius;
        float z2 = cosf( r0 * angle ) * radius;
        float y2 = ( h1 / (numSteps-1.0f)) * height;

        float x3 = sinf( r1 * angle ) * radius;
        float z3 = cosf( r1 * angle ) * radius;
        float y3 = ( h1 / (numSteps-1.0f)) * height;

        float anglex = (float)TWOPI / ((float)numRings);
        ShapePoint tp0 = {x0, y0, z0,     x0, 0,  z0,   ((float)h0 / (numSteps-1.0f)), abs(sinf( (float)r0 * anglex ))      };
        ShapePoint tp1 = {x1, y1, z1,     x1, 0,  z1,   ((float)h0 / (numSteps-1.0f)), abs(sinf( (float)r1 * anglex ))      };
        ShapePoint tp2 = {x2, y2, z2,     x2, 0,  z2,   ((float)h1 / (numSteps-1.0f)), abs(sinf( (float)r0 * anglex ))      };
        ShapePoint tp3 = {x3, y3, z3,     x3, 0,  z3,   ((float)h1 / (numSteps-1.0f)), abs(sinf( (float)r1 * anglex ))      };

        outTris->push_back( tp0 );
        outTris->push_back( tp1 );
        outTris->push_back( tp2 );

        outTris->push_back( tp2 );
        outTris->push_back( tp1 );
        outTris->push_back( tp3 );

    }// End for r
  }// End for h

}// End GenerateTube(..)


/***************************************************************************/
/*                                                                         */
/* Name: torus(..)                                                         */
/* Date: 07-08-2004                                                        */
/* Orig: benjamin kenwright                                                */
/*                                                                         */
/* Desc: Generates and draws a torus shape for debugging.                  */
/* A simple asci sketch below to show how you effect the number of turns   */
/* about the radius.and the number of turns about the column that makes    */
/* up the tube of the donut.                                               */
/*                                                                         */
/***************************************************************************/
/*
       Front View             Size View
         _ _ _ _                -----                 
       /        \               | | |             
     /            \             | | |              
   /                \           -----                 
  |                  |          | | |                 
  |                  |          | | |                 
  |                  |          | | |                  
  |                  |          -----                    
   \                /           | | |                 
     \            /             | | |           
       \ _ _ _ _/               -----            
                                                  
        numt=8                  numc=3  

*/
/***************************************************************************/



inline
void GenerateTorus(int numc, int numt, vector<ShapePoint>* tvecs)
{
  // Quads not triangles

#if 0
    double x,  y,  z;  // position
    double nx, ny, nz; // normal
    double u,  v;      // texture coords

    for (int i = 0; i < numc; i++) 
    {
        for (int j = 0; j <= numt; j++) 
        {
            for (int k = 1; k >= 0; k--) 
            {
                double s = (i + k) % numc + 0.5;
                double t = j % numt;
                // Alter the 0.3 to larger values to get larger
                // or smaller tube of your donut :)
                x = (1+.3*cos(s*TWOPI/numc))*cos(t*TWOPI/numt);
                y = (1+.3*cos(s*TWOPI/numc))*sin(t*TWOPI/numt);
                z = .3 * sin(s*TWOPI / numc);

                nx = cos(s*TWOPI/numc)*cos(t*TWOPI/numt);
                ny = (1+.3*cos(s*TWOPI/numc))*sin(t*TWOPI/numt);
                nz = sin(s*TWOPI / numc);

                u = (i + k) / (float)numc;
                v = t / (float)numt;
                
                TorusPoint tp = {(float)x, (float)y, (float)z, 
                                 (float)nx,(float)ny,(float)nz,
                                 (float)u, (float)v};
                tvecs->push_back( tp );
            }//End for k
        }//End for j
    }//End for i
#endif
}//End Torus(..)









