/****************************************************************/
/*  
     ***   ***  ***  *** ***     *** ***   ******  ****  ***
     ***   ***  ***  *** ***     *** **   ***  *** ***** ***
     ***   ***  ***  *** ***     *****    ******** *** *****
      *** ***   ***  *** ******* *** **   ***  *** ***  ****
       *****     *****   ******* *** ***  ***  *** ***   ***
	   
	 Vulkan Graphics API: 11 Steps

     Minimalistic examples to build upon and experiment with 
	 to develop an understanding of the underlying API

	   
/*
/****************************************************************/

// Fragement Shader or sometimes called the `pixel shader'

// shader version
#version 450

#extension GL_ARB_separate_shader_objects : enable
#extension GL_ARB_shading_language_420pack : enable


layout(binding = 1) uniform sampler2D texSampler;

// input (from our previous shader stage)
layout ( location = 0 ) in vec4 inPos;
layout ( location = 1 ) in vec4 inColor;
layout ( location = 2 ) in vec2 inTexCoord;
layout ( location = 3 ) in vec3 inNormal;


// final screen output color
layout ( location = 0 ) out vec4 outFragColor;

// set the light position (modified to be passed in 
// from the program via a uniform)
vec3 lightPosition = vec3(1,1,3);

// shader entry point
void main() 
{
  vec3 N = normalize(inNormal);
  vec3 L = normalize(lightPosition - inPos.xyz);   
  vec3 E = normalize(-inPos.xyz); // you are in Eye Coordinates, so EyePos is (0,0,0)  
  vec3 R = normalize(-reflect(L,N));  
   
  //calculate Ambient Term:  
  float Iamb = 0.2;    
  
  //calculate Diffuse Term:
  float Idiff = max(dot(N,L), 0.0);
  Idiff =  clamp(Idiff, 0.0, 1.0);
     
  // calculate Specular Term:
  float shininess = 16.0;
  float Ispec = pow(max(dot(R,E),0.0),shininess);
  Ispec = clamp(Ispec, 0.0, 1.0); 


  // get the color from the texture
  vec4 texColor = texture(texSampler, inTexCoord);

  // texture color with lighting
  outFragColor = vec4((Iamb + Idiff) * texColor + Ispec*(0.5,0.5,0.5,1.0) );

  // vertex color
  // outFragColor = inColor;

  // constant color (debugging)
  // outFragColor = vec4(1,0,0,1);
}// End main(..)