/****************************************************************/
/*  
     ***   ***  ***  *** ***     *** ***   ******  ****  ***
     ***   ***  ***  *** ***     *** **   ***  *** ***** ***
     ***   ***  ***  *** ***     *****    ******** *** *****
      *** ***   ***  *** ******* *** **   ***  *** ***  ****
       *****     *****   ******* *** ***  ***  *** ***   ***
	   
	 Vulkan Graphics API: 11 Steps

     Minimalistic examples to build upon and experiment with 
	 to develop an understanding of the underlying API

	   
/*
/****************************************************************/

// Tessellation Evaluation Shader

#version 450

#extension GL_ARB_separate_shader_objects : enable
#extension GL_ARB_shading_language_420pack : enable

// layout (triangles) in;
layout(quads) in;

// input into the shader
layout ( location = 0 ) in  vec4 inPos[];
layout ( location = 1 ) in  vec4 inColor[];
layout ( location = 2 ) in  vec2 inTexCoords[];
layout ( location = 3 ) in  vec3 inNormal[];


// output to next stage
layout ( location = 0 ) out vec4 outPos;
layout ( location = 1 ) out vec4 outColor;
layout ( location = 2 ) out vec2 outTexCoords;
layout ( location = 3 ) out vec3 outNormal;



void main(void)
{
	// Interpolate positions
    vec4 pos1   = mix(gl_in[0].gl_Position, gl_in[1].gl_Position, gl_TessCoord.x);
    vec4 pos2   = mix(gl_in[3].gl_Position, gl_in[2].gl_Position, gl_TessCoord.x);
    gl_Position = mix(pos1, pos2, gl_TessCoord.y);

	vec4 opos1  = mix(inPos[0], inPos[1], gl_TessCoord.x);
    vec4 opos2  = mix(inPos[3], inPos[2], gl_TessCoord.x);
    outPos      = mix(opos1, opos2, gl_TessCoord.y);

    vec4 col1  = mix(inColor[0], inColor[1], gl_TessCoord.x);
    vec4 col2  = mix(inColor[3], inColor[2], gl_TessCoord.x);
    outColor   = mix(col1, col2, gl_TessCoord.y);

	vec2 uv1  = mix(inTexCoords[0], inTexCoords[1], gl_TessCoord.x);
    vec2 uv2  = mix(inTexCoords[3], inTexCoords[2], gl_TessCoord.x);
    outTexCoords = mix(uv1, uv2, gl_TessCoord.y);

	vec3 n1  = mix(inNormal[0], inNormal[1], gl_TessCoord.x);
    vec3 n2  = mix(inNormal[3], inNormal[2], gl_TessCoord.x);
    outNormal = mix(n1, n2, gl_TessCoord.y);

}// End main(..)