/****************************************************************/
/*  
     ***   ***  ***  *** ***     *** ***   ******  ****  ***
     ***   ***  ***  *** ***     *** **   ***  *** ***** ***
     ***   ***  ***  *** ***     *****    ******** *** *****
      *** ***   ***  *** ******* *** **   ***  *** ***  ****
       *****     *****   ******* *** ***  ***  *** ***   ***
	   
	 Vulkan Graphics API: 11 Steps

     Minimalistic examples to build upon and experiment with 
	 to develop an understanding of the underlying API

	   
/*
/****************************************************************/

// Vertex Shader

// shader version
#version 450

#extension GL_ARB_separate_shader_objects : enable
#extension GL_ARB_shading_language_420pack : enable


// input from your program
layout ( location = 0 ) in  vec4 inPos;
layout ( location = 1 ) in  vec3 inColor;
layout ( location = 2 ) in  vec2 inTexCoords;
layout ( location = 3 ) in  vec3 inNormal;


// input matrix transforms (3D) passed in from our program
layout ( binding = 0 ) uniform buff 
{
  mat4 inProjectionMatrix;
  mat4 inViewMatrix;
  mat4 inModelMatrix;
};

// output to next stage
layout ( location = 0 ) out vec4 outPos;
layout ( location = 1 ) out vec4 outColor;
layout ( location = 2 ) out vec2 outTexCoords;
layout ( location = 3 ) out vec3 outNormal;

out gl_PerVertex
{
	vec4 gl_Position;
};

// shader entry point
void main() 
{	  
  // combine the matrices
  mat4 mvp = inModelMatrix * inViewMatrix * inProjectionMatrix;

  // transform position by matrices
  gl_Position = vec4(inPos.xyz, 1) * mvp;

  outPos = vec4(vec4(inPos.xyz, 1) * inModelMatrix);

  // pass input colour to the next stage
  outColor = vec4(inColor, 1.0);

  // pass input texture coordinates onto the fragment shader
  outTexCoords = inTexCoords;

  // pass input normals onto the fragment shader
  outNormal = mat3(transpose(inverse(inModelMatrix))) * inNormal; 

}// End main(..)